/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2015-2016 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "surfaceReader.H"

// * * * * * * * * * * * * * Static Member Functions * * * * * * * * * * * * //

Foam::autoPtr<Foam::surfaceReader> Foam::surfaceReader::New
(
    const word& readerType,
    const fileName& fName
)
{
    fileNameConstructorTable::iterator cstrIter =
        fileNameConstructorTablePtr_->find(readerType);

    if (cstrIter == fileNameConstructorTablePtr_->end())
    {
        FatalErrorInFunction
            << "Unknown reader type \"" << readerType << "\"\n\n"
            << "Valid reader types: "
            << fileNameConstructorTablePtr_->sortedToc() << nl
            << exit(FatalError);
    }

    return autoPtr<surfaceReader>(cstrIter()(fName));
}


// ************************************************************************* //
